<?php

abstract class LocationCollector {
  protected $filename;

  public function __construct($filename) {
    $this->filename = $filename;
  }

  public abstract function collect();
}

class TSVCollector extends LocationCollector {
  public function collect() {
    $ret = array();
    foreach(file($this->filename) as $line) {
      $ret []= explode("\t", rtrim($line));
    }
    return $ret;
  }
}

class HookCollector {
  
  /**
   * Name of the hook which will be called by collect().
   *
   * @var string
   */
  protected $hookname = 'collect_locations';

  protected function save($name, $data) {
    cache_set($name, $data, LDHM_TABLE);
  }

  protected function processLocation(LocationCollector $location) {
    $hookinfo = $location->collect();
    foreach($hookinfo as $h) {
      $hookname = array_shift($h);
      $hook = new Hook();
      $hook->set_return_type(array_shift($h));
      while(count($h) > 0) {
        $type = array_shift($h);
        $name = array_shift($h);
        $reference = FALSE;
        if($name{0} == '&') {
          $name = ltrim($name, '&');
          $reference = TRUE;
        }
        $hp = new HookParameter($name, $type, $reference);
        $hook->add_parameter($hp);
      }
      $this->save($hookname, $hook);
    }
  }

  public function collect() {
    $locations = module_invoke_all($this->hookname);
    foreach($locations as $l) {
      $this->processLocation($l);
    }
  }

}

function lambda_db_hookmanager_collect_locations() {
  return array(new TSVCollector(drupal_get_path('module', 'lambda_db_hookmanager').'/corehooks.tsv'));
}

function collectHooks() {
  $collector = new HookCollector();
  $collector->collect();
}
